@@ -34,16 +34,12 @@ namespace primesieve {
 uint64_t nth_prime(int64_t n, uint64_t start)
 {
   ParallelSieve ps;
-  ps.setSieveSize(get_sieve_size());
-  ps.setNumThreads(get_num_threads());
   return ps.nthPrime(n, start);
 }
 
 uint64_t count_primes(uint64_t start, uint64_t stop)
 {
   ParallelSieve ps;
-  ps.setSieveSize(get_sieve_size());
-  ps.setNumThreads(get_num_threads());
   ps.sieve(start, stop, COUNT_PRIMES);
   return ps.getCount(0);
 }
@@ -51,8 +47,6 @@ uint64_t count_primes(uint64_t start, uint64_t stop)
 uint64_t count_twins(uint64_t start, uint64_t stop)
 {
   ParallelSieve ps;
-  ps.setSieveSize(get_sieve_size());
-  ps.setNumThreads(get_num_threads());
   ps.sieve(start, stop, COUNT_TWINS);
   return ps.getCount(1);
 }
@@ -60,8 +54,6 @@ uint64_t count_twins(uint64_t start, uint64_t stop)
 uint64_t count_triplets(uint64_t start, uint64_t stop)
 {
   ParallelSieve ps;
-  ps.setSieveSize(get_sieve_size());
-  ps.setNumThreads(get_num_threads());
   ps.sieve(start, stop, COUNT_TRIPLETS);
   return ps.getCount(2);
 }
@@ -69,8 +61,6 @@ uint64_t count_triplets(uint64_t start, uint64_t stop)
 uint64_t count_quadruplets(uint64_t start, uint64_t stop)
 {
   ParallelSieve ps;
-  ps.setSieveSize(get_sieve_size());
-  ps.setNumThreads(get_num_threads());
   ps.sieve(start, stop, COUNT_QUADRUPLETS);
   return ps.getCount(3);
 }
@@ -78,8 +68,6 @@ uint64_t count_quadruplets(uint64_t start, uint64_t stop)
 uint64_t count_quintuplets(uint64_t start, uint64_t stop)
 {
   ParallelSieve ps;
-  ps.setSieveSize(get_sieve_size());
-  ps.setNumThreads(get_num_threads());
   ps.sieve(start, stop, COUNT_QUINTUPLETS);
   return ps.getCount(4);
 }
@@ -87,8 +75,6 @@ uint64_t count_quintuplets(uint64_t start, uint64_t stop)
 uint64_t count_sextuplets(uint64_t start, uint64_t stop)
 {
   ParallelSieve ps;
-  ps.setSieveSize(get_sieve_size());
-  ps.setNumThreads(get_num_threads());
   ps.sieve(start, stop, COUNT_SEXTUPLETS);
   return ps.getCount(5);
 }
@@ -96,42 +82,36 @@ uint64_t count_sextuplets(uint64_t start, uint64_t stop)
 void print_primes(uint64_t start, uint64_t stop)
 {
   PrimeSieve ps;
-  ps.setSieveSize(get_sieve_size());
   ps.sieve(start, stop, PRINT_PRIMES);
 }
 
 void print_twins(uint64_t start, uint64_t stop)
 {
   PrimeSieve ps;
-  ps.setSieveSize(get_sieve_size());
   ps.sieve(start, stop, PRINT_TWINS);
 }
 
 void print_triplets(uint64_t start, uint64_t stop)
 {
   PrimeSieve ps;
-  ps.setSieveSize(get_sieve_size());
   ps.sieve(start, stop, PRINT_TRIPLETS);
 }
 
 void print_quadruplets(uint64_t start, uint64_t stop)
 {
   PrimeSieve ps;
-  ps.setSieveSize(get_sieve_size());
   ps.sieve(start, stop, PRINT_QUADRUPLETS);
 }
 
 void print_quintuplets(uint64_t start, uint64_t stop)
 {
   PrimeSieve ps;
-  ps.setSieveSize(get_sieve_size());
   ps.sieve(start, stop, PRINT_QUINTUPLETS);
 }
 
 void print_sextuplets(uint64_t start, uint64_t stop)
 {
   PrimeSieve ps;
-  ps.setSieveSize(get_sieve_size());
   ps.sieve(start, stop, PRINT_SEXTUPLETS);
 }
 
@@ -158,9 +138,9 @@ std::string primesieve_version()
   return PRIMESIEVE_VERSION;
 }
 
-void set_sieve_size(int kilobytes)
+void set_sieve_size(int size)
 {
-  sieve_size = inBetween(8, kilobytes, 4096);
+  sieve_size = inBetween(8, size, 4096);
   sieve_size = floorPow2(sieve_size);
 }
 
@@ -170,34 +150,35 @@ int get_sieve_size()
   if (sieve_size)
     return sieve_size;
 
-  size_t l1CacheSize = cpuInfo.l1CacheSize();
-  size_t l2CacheSize = cpuInfo.l2CacheSize();
-
-  // convert to kilobytes
-  l1CacheSize /= 1024;
-  l2CacheSize /= 1024;
-
-  // check if each CPU core has a private L2 cache
-  if (cpuInfo.hasL2Cache() &&
-      cpuInfo.privateL2Cache() &&
-      l2CacheSize > l1CacheSize)
+  // Shared CPU caches are usually slow. Hence we only use
+  // the L2 cache for sieving if each physical CPU core
+  // has a private L2 cache. Also we only use half of the
+  // L2 cache for the sieve array so that other important
+  // data structures can also fit into the L2 cache.
+  if (cpuInfo.hasPrivateL2Cache())
   {
-    l2CacheSize = inBetween(32, l2CacheSize, 4096);
-    l2CacheSize = floorPow2(l2CacheSize);
-    return (int) l2CacheSize;
+    // convert bytes to KiB
+    size_t size = cpuInfo.l2CacheSize() >> 10;
+    size = size - 1;
+    size = inBetween(32, size, 4096);
+    size = floorPow2(size);
+    return (int) size;
+  }
+  else if (cpuInfo.hasL1Cache())
+  {
+    // convert bytes to KiB
+    size_t size = cpuInfo.l1CacheSize() >> 10;
+    size = inBetween(8, size, 4096);
+    size = floorPow2(size);
+    return (int) size;
   }
   else
   {
-    if (!cpuInfo.hasL1Cache())
-      l1CacheSize = 32;
-
-    // if the CPU does not have an L2 cache or if the
-    // cache is shared between all CPU cores we
-    // set the sieve size to the CPU's L1 cache size
-
-    l1CacheSize = inBetween(8, l1CacheSize, 4096);
-    l1CacheSize = floorPow2(l1CacheSize);
-    return (int) l1CacheSize;
+    // default sieve size in KiB
+    size_t size = 32;
+    size = inBetween(8, size, 4096);
+    size = floorPow2(size);
+    return (int) size;
   }
 }
 
