@@ -12,13 +12,16 @@
 #include "cmdoptions.hpp"
 
 #include <primesieve/calculator.hpp>
+#include <primesieve/CpuInfo.hpp>
 #include <primesieve/PrimeSieve.hpp>
 #include <primesieve/primesieve_error.hpp>
 
-#include <string>
-#include <map>
 #include <cstddef>
+#include <cstdlib>
+#include <iostream>
+#include <map>
 #include <stdint.h>
+#include <string>
 
 void help();
 void version();
@@ -35,6 +38,7 @@ struct Option
   string str;
   string opt;
   string val;
+
   template <typename T>
   T getValue() const
   {
@@ -47,8 +51,9 @@ struct Option
 enum OptionID
 {
   OPTION_COUNT,
+  OPTION_CPU_INFO,
   OPTION_HELP,
-  OPTION_NTHPRIME,
+  OPTION_NTH_PRIME,
   OPTION_NO_STATUS,
   OPTION_NUMBER,
   OPTION_DISTANCE,
@@ -65,10 +70,12 @@ map<string, OptionID> optionMap =
 {
   { "-c",          OPTION_COUNT },
   { "--count",     OPTION_COUNT },
+  { "--cpu-info",  OPTION_CPU_INFO },
   { "-h",          OPTION_HELP },
   { "--help",      OPTION_HELP },
-  { "-n",          OPTION_NTHPRIME },
-  { "--nthprime",  OPTION_NTHPRIME },
+  { "-n",          OPTION_NTH_PRIME },
+  { "--nthprime",  OPTION_NTH_PRIME },
+  { "--nth-prime", OPTION_NTH_PRIME },
   { "--no-status", OPTION_NO_STATUS },
   { "--number",    OPTION_NUMBER },
   { "-d",          OPTION_DISTANCE },
@@ -131,6 +138,19 @@ void optionCount(Option& opt,
   }
 }
 
+void optionDistance(Option& opt,
+                    CmdOptions& opts)
+{
+  uint64_t start = 0;
+  uint64_t val = opt.getValue<uint64_t>();
+  auto& numbers = opts.numbers;
+
+  if (!numbers.empty())
+    start = numbers[0];
+
+  numbers.push_back(start + val);
+}
+
 /// e.g. "--thread=4" -> return "--thread"
 string getOption(const string& str)
 {
@@ -174,6 +194,81 @@ Option makeOption(const string& str)
   return opt;
 }
 
+void optionCpuInfo()
+{
+  const CpuInfo cpu;
+
+  if (cpu.hasCpuName())
+    cout << cpu.cpuName() << endl;
+  else
+    cout << "CPU name: unknown" << endl;
+
+  if (cpu.hasCpuCores())
+    cout << "Number of cores: " << cpu.cpuCores() << endl;
+  else
+    cout << "Number of cores: unknown" << endl;
+
+  if (cpu.hasCpuThreads())
+    cout << "Number of threads: " << cpu.cpuThreads() << endl;
+  else
+    cout << "Number of threads: unknown" << endl;
+
+  if (cpu.hasThreadsPerCore())
+    cout << "Threads per core: " << cpu.threadsPerCore() << endl;
+  else
+    cout << "Threads per core: unknown" << endl;
+
+  if (cpu.hasL1Cache())
+    cout << "L1 cache size: " << cpu.l1CacheSize() / (1 << 10) << " KiB" << endl;
+
+  if (cpu.hasL2Cache())
+    cout << "L2 cache size: " << cpu.l2CacheSize() / (1 << 10) << " KiB" << endl;
+
+  if (cpu.hasL3Cache())
+    cout << "L3 cache size: " << cpu.l3CacheSize() / (double) (1 << 20) << " MiB" << endl;
+
+  if (cpu.hasL1Cache())
+  {
+    if (!cpu.hasL1Sharing())
+      cout << "L1 cache sharing: unknown" << endl;
+    else
+      cout << "L1 cache sharing: " << cpu.l1Sharing()
+           << ((cpu.l1Sharing() > 1) ? " threads" : " thread") << endl;
+  }
+
+  if (cpu.hasL2Cache())
+  {
+    if (!cpu.hasL2Sharing())
+      cout << "L2 cache sharing: unknown" << endl;
+    else
+      cout << "L2 cache sharing: " << cpu.l2Sharing()
+           << ((cpu.l2Sharing() > 1) ? " threads" : " thread") << endl;
+  }
+
+  if (cpu.hasL3Cache())
+  {
+    if (!cpu.hasL3Sharing())
+      cout << "L3 cache sharing: unknown" << endl;
+    else
+      cout << "L3 cache sharing: " << cpu.l3Sharing()
+           << ((cpu.l3Sharing() > 1) ? " threads" : " thread") << endl;
+  }
+
+  if (!cpu.hasL1Cache() &&
+      !cpu.hasL2Cache() &&
+      !cpu.hasL3Cache())
+  {
+    cout << "L1 cache size: unknown" << endl;
+    cout << "L2 cache size: unknown" << endl;
+    cout << "L3 cache size: unknown" << endl;
+    cout << "L1 cache sharing: unknown" << endl;
+    cout << "L2 cache sharing: unknown" << endl;
+    cout << "L3 cache sharing: unknown" << endl;
+  }
+
+  exit(0);
+}
+
 } // namespace
 
 CmdOptions parseOptions(int argc, char* argv[])
@@ -187,15 +282,16 @@ CmdOptions parseOptions(int argc, char* argv[])
     switch (optionMap[opt.opt])
     {
       case OPTION_COUNT:     optionCount(opt, opts); break;
+      case OPTION_CPU_INFO:  optionCpuInfo(); break;
+      case OPTION_DISTANCE:  optionDistance(opt, opts); break;
       case OPTION_PRINT:     optionPrint(opt, opts); break;
       case OPTION_SIZE:      opts.sieveSize = opt.getValue<int>(); break;
       case OPTION_THREADS:   opts.threads = opt.getValue<int>(); break;
       case OPTION_QUIET:     opts.quiet = true; break;
-      case OPTION_NTHPRIME:  opts.nthPrime = true; break;
+      case OPTION_NTH_PRIME: opts.nthPrime = true; break;
       case OPTION_NO_STATUS: opts.status = false; break;
       case OPTION_TIME:      opts.time = true; break;
       case OPTION_NUMBER:    opts.numbers.push_back(opt.getValue<uint64_t>()); break;
-      case OPTION_DISTANCE:  opts.numbers.push_back(opt.getValue<uint64_t>() + opts.numbers[0]); break;
       case OPTION_VERSION:   version(); break;
       case OPTION_HELP:      help(); break;
     }
